@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -15,6 +15,12 @@
 #include "Generic.h"
 #include "Store.h"
 
+/*
+ * squidaio_ctrl_t uses explicit alloc()/freeOne() allocators
+ * XXX: convert to MEMPROXY_CLASS() API
+ */
+#include "mem/Pool.h"
+
 AIOCounts squidaio_counts;
 
 typedef struct squidaio_unlinkq_t {
