@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -16,13 +16,13 @@ class ACLChecklist;
 class ACLFilledChecklist;
 class ACLList;
 
-class AclAddress;
 class AclDenyInfoList;
 class AclSizeLimit;
 
 namespace Acl
 {
 
+class Address;
 class InnerNode;
 class NotNode;
 class AndNode;
